@@ -3,6 +3,7 @@ from __future__ import annotations
 import datetime
 import enum
 import re
+import warnings
 from typing import Any, Optional, Union, cast
 from uuid import uuid4
 
@@ -56,6 +57,9 @@ class ValueObject:
     def __repr__(self) -> str:
         return f"{self.__class__.__name__}({self.value!r})"
 
+    def __str__(self) -> str:
+        return str(self.value)
+
 
 class Flag(ValueObject):
     def __init__(self, value: bool, /):
@@ -249,9 +253,9 @@ class JobName(ValueObject):
                 f"{self.__class__.__name__} value is required, but got None."
             )
         elif isinstance(value, str):
-            if len(value) < 3 or len(value) >= 100:
+            if len(value) < 3 or len(value) >= 200:
                 raise ValueError(
-                    f"{self.__class__.__name__} must be between 3 and 100 characters long, but got "
+                    f"{self.__class__.__name__} must be between 3 and 200 characters long, but got "
                     f"{value!r}."
                 )
         else:
@@ -269,9 +273,9 @@ class SecondsBetweenRefreshes(ValueObject):
                 f"{self.__class__.__name__} value is required, but got None."
             )
         elif isinstance(value, int):
-            if value < 300:
+            if value < 0:
                 raise ValueError(
-                    f"If a {self.__class__.__name__} must be at least 300 seconds, but got {value!r}."
+                    f"If a {self.__class__.__name__} must be positive, but got {value!r}."
                 )
         else:
             raise TypeError(
@@ -288,9 +292,9 @@ class TestName(ValueObject):
                 f"{self.__class__.__name__} value is required, but got None."
             )
         elif isinstance(value, str):
-            if len(value) < 3 or len(value) > 100:
+            if len(value) < 3 or len(value) > 200:
                 raise ValueError(
-                    f"{self.__class__.__name__} must be between 3 and 100 characters long, "
+                    f"{self.__class__.__name__} must be between 3 and 200 characters long, "
                     f"but got {value!r}."
                 )
         else:
@@ -478,8 +482,25 @@ class LogLevel(ValueObject):
             raise ValueError(f"{self.value} is not a LogLevelOption value.")
 
 
-class LogMessage(_NonEmptyStr):
-    ...
+class LogMessage(ValueObject):
+    def __init__(self, value: str, /):
+        if not value:
+            raise ValueError(
+                f"{self.__class__.__name__} value is required, but got {value!r}."
+            )
+        elif isinstance(value, str):
+            if len(value) > 2000:
+                warnings.warn(
+                    f"{self.__class__.__name__} must be <= 2000 characters long, but the message "
+                    f"is {len(value)}. It has been truncated to fit."
+                )
+                value = value[-2000:]
+        else:
+            raise TypeError(
+                f"{self.__class__.__name__} expects a str, but got {value!r}"
+            )
+
+        super().__init__(value)
 
 
 class SecondsSinceLastRefresh(ValueObject):
