@@ -1,10 +1,18 @@
+import itertools
 from typing import List, Protocol, runtime_checkable
 
 import typing
 
 from lime_etl.adapters import timestamp_adapter
-from lime_etl.domain import batch_delta
-from lime_etl.domain import batch, job_result, job_spec, value_objects
+from lime_etl.domain import (
+    batch_delta,
+    batch,
+    exceptions,
+    job_dependency_errors,
+    job_result,
+    job_spec,
+    value_objects,
+)
 from lime_etl.services import (
     batch_logging_service,
     job_runner,
@@ -18,7 +26,7 @@ class BatchRunner(Protocol):
         self,
         uow: unit_of_work.UnitOfWork,
         batch_id: value_objects.UniqueId,
-        jobs: List[job_spec.JobSpec],
+        jobs: typing.Collection[job_spec.JobSpec],
         batch_logger: batch_logging_service.BatchLoggingService,
         ts_adapter: timestamp_adapter.TimestampAdapter,
     ) -> batch.Batch:
@@ -27,13 +35,21 @@ class BatchRunner(Protocol):
 
 def run(
     uow: unit_of_work.UnitOfWork,
-    jobs: typing.Iterable[job_spec.JobSpec],
+    jobs: typing.Collection[job_spec.JobSpec],
     ts_adapter: timestamp_adapter.TimestampAdapter,
 ) -> batch_delta.BatchDelta:
     batch_id = value_objects.UniqueId.generate()
     batch_logger = batch_logging_service.DefaultBatchLoggingService(
         uow=uow, batch_id=batch_id
     )
+    dep_results = check_dependencies(jobs)
+    if dep_results:
+        e = exceptions.DependencyErrors(dep_results)
+
+        batch_logger.log_error(
+            value_objects.LogMessage("\n".join(str(e) for e in sorted(dep_results)))
+        )
+        raise exceptions.DependencyErrors(dep_results)
 
     try:
         with uow:
@@ -60,7 +76,9 @@ def run(
             ts_adapter=ts_adapter,
         )
         uow.batches.update(result)
-        batch_logger.log_info(value_objects.LogMessage(f"Batch [{batch_id.value}] finished."))
+        batch_logger.log_info(
+            value_objects.LogMessage(f"Batch [{batch_id.value}] finished.")
+        )
         return batch_delta.BatchDelta(
             current_results=result,
             previous_results=previous_results,
@@ -70,6 +88,47 @@ def run(
         raise
 
 
+def check_dependencies(
+    jobs: typing.Collection[job_spec.JobSpec], /
+) -> typing.Set[job_dependency_errors.JobDependencyErrors]:
+    job_names = {job.job_name for job in jobs}
+    unresolved_dependencies_by_table = {
+        job.job_name: set(dep for dep in job.dependencies if dep not in job_names)
+        for job in jobs
+        if any(dep not in job_names for dep in job.dependencies)
+    }
+    unresolved_dependencies = {
+        dep for dep_grp in unresolved_dependencies_by_table.values() for dep in dep_grp
+    }
+
+    job_names_seen_so_far: typing.List[value_objects.JobName] = []
+    jobs_out_of_order_by_table: typing.Dict[
+        value_objects.JobName, typing.Set[value_objects.JobName]
+    ] = dict()
+    for job in jobs:
+        job_names_seen_so_far.append(job.job_name)
+        job_deps_out_of_order = []
+        for dep in job.dependencies:
+            if dep not in job_names_seen_so_far and dep not in unresolved_dependencies:
+                job_deps_out_of_order.append(dep)
+        if job_deps_out_of_order:
+            jobs_out_of_order_by_table[job.job_name] = set(job_deps_out_of_order)
+
+    return {
+        job_dependency_errors.JobDependencyErrors(
+            job_name=job_name,
+            missing_dependencies=frozenset(unresolved_dependencies_by_table.get(job_name, set())),
+            jobs_out_of_order=frozenset(jobs_out_of_order_by_table.get(job_name, set())),
+        )
+        for job_name in set(
+            itertools.chain(
+                unresolved_dependencies_by_table.keys(),
+                jobs_out_of_order_by_table.keys(),
+            )
+        )
+    }
+
+
 def _run_batch(
     batch_logger: batch_logging_service.BatchLoggingService,
     uow: unit_of_work.UnitOfWork,
