@@ -64,13 +64,11 @@ def default_job_runner(
                     f"The following dependencies failed to execute: {exceptions}"
                 )
             else:
-                result = _run_with_retry(
+                result = _run_jobs_with_tests(
                     batch_id=batch_id,
                     job_id=job_id,
                     job=job,
                     logger=logger,
-                    retries_so_far=0,
-                    max_retries=job.max_retries.value,
                     ts_adapter=ts_adapter,
                     uow=uow,
                 )
@@ -81,14 +79,7 @@ def default_job_runner(
                 )
                 return result
     except Exception as e:
-        logger.log_error(
-            value_objects.LogMessage(
-                (
-                    f"An error occurred while running the job [{job.job_name.value}]:"
-                    f"\n{traceback.format_exc(10)}"
-                )
-            ),
-        )
+        logger.log_error(value_objects.LogMessage(traceback.format_exc(10)))
         return job_result.JobResult(
             id=job_id,
             batch_id=batch_id,
@@ -100,41 +91,123 @@ def default_job_runner(
         )
 
 
-def _run_with_retry(
+def _run_jobs_with_tests(
     batch_id: value_objects.UniqueId,
     job_id: value_objects.UniqueId,
     job: job_spec.JobSpec,
-    max_retries: int,
-    retries_so_far: int,
     logger: job_logging_service.JobLoggingService,
     ts_adapter: timestamp_adapter.TimestampAdapter,
     uow: unit_of_work.UnitOfWork,
 ) -> job_result.JobResult:
+    result, execution_millis = _run_with_retry(
+        job=job,
+        logger=logger,
+        retries_so_far=0,
+        max_retries=job.max_retries.value,
+        uow=uow,
+    )
+    if result.is_success:
+        logger.log_info(
+            value_objects.LogMessage(f"[{job.job_name.value}] finished successfully.")
+        )
+        logger.log_info(
+            value_objects.LogMessage(f"Running the tests for [{job.job_name.value}]...")
+        )
+        test_start_time = datetime.datetime.now()
+        if isinstance(job, job_spec.AdminJobSpec):
+            test_results = job.test(logger=logger, uow=uow)
+        elif isinstance(job, job_spec.ETLJobSpec):
+            test_results = job.test(logger=logger)
+        else:
+            raise ValueError(f"Expected either an AdminJobSpec or an ETLJobSpec, but got {job!r}")
+        test_execution_millis = int(
+            (datetime.datetime.now() - test_start_time).total_seconds() * 1000
+        )
+
+        if test_results:
+            tests_passed = sum(
+                1 for test_result in test_results if test_result.test_passed
+            )
+            tests_failed = sum(
+                1 for test_result in test_results if test_result.test_failed
+            )
+            logger.log_info(
+                value_objects.LogMessage(
+                    f"{job.job_name.value} test results: {tests_passed=}, {tests_failed=}"
+                )
+            )
+            full_test_results: typing.FrozenSet[
+                job_test_result.JobTestResult
+            ] = frozenset(
+                job_test_result.JobTestResult(
+                    id=value_objects.UniqueId.generate(),
+                    job_id=job_id,
+                    test_name=test_result.test_name,
+                    test_success_or_failure=test_result.test_success_or_failure,
+                    execution_millis=value_objects.ExecutionMillis(
+                        test_execution_millis
+                    ),
+                    execution_success_or_failure=value_objects.Result.success(),
+                    ts=ts_adapter.now(),
+                )
+                for test_result in test_results
+            )
+        else:
+            logger.log_info(
+                value_objects.LogMessage("The job test method returned no results.")
+            )
+            full_test_results = frozenset()
+    else:
+        logger.log_info(
+            value_objects.LogMessage(
+                f"An exception occurred while running [{job.job_name.value}]: "
+                f"{result.failure_message}."
+            )
+        )
+        full_test_results = frozenset()
+
+    ts = ts_adapter.now()
+    return job_result.JobResult(
+        id=job_id,
+        batch_id=batch_id,
+        job_name=job.job_name,
+        test_results=full_test_results,
+        execution_millis=value_objects.ExecutionMillis(execution_millis),
+        execution_success_or_failure=value_objects.Result.success(),
+        ts=ts,
+    )
+
+
+def _run_with_retry(
+    job: job_spec.JobSpec,
+    max_retries: int,
+    retries_so_far: int,
+    logger: job_logging_service.JobLoggingService,
+    uow: unit_of_work.UnitOfWork,
+) -> typing.Tuple[value_objects.Result, int]:
     # noinspection PyBroadException
     try:
-        return _run_job_with_tests(
-            batch_id=batch_id,
-            job_id=job_id,
+        start_time = datetime.datetime.now()
+        result = _run(
             job=job,
             logger=logger,
-            ts_adapter=ts_adapter,
             uow=uow,
         )
+        end_time = datetime.datetime.now()
+        return result, int((end_time - start_time).total_seconds() * 1000)
     except:
+        logger.log_error(value_objects.LogMessage(traceback.format_exc(10)))
         if max_retries > retries_so_far:
             logger.log_info(
                 value_objects.LogMessage(
-                    f"Running retry {retries_so_far} or {max_retries}..."
+                    f"Running retry {retries_so_far} of {max_retries}..."
                 )
             )
             return _run_with_retry(
-                batch_id=batch_id,
-                job_id=job_id,
                 job=job,
                 max_retries=max_retries,
                 retries_so_far=retries_so_far + 1,
                 logger=logger,
-                ts_adapter=ts_adapter,
                 uow=uow,
             )
         else:
@@ -146,76 +219,16 @@ def _run_with_retry(
             raise
 
 
-def _run_job_with_tests(
-    batch_id: value_objects.UniqueId,
-    job_id: value_objects.UniqueId,
+def _run(
     job: job_spec.JobSpec,
     logger: job_logging_service.JobLoggingService,
-    ts_adapter: timestamp_adapter.TimestampAdapter,
     uow: unit_of_work.UnitOfWork,
-) -> job_result.JobResult:
-    start_time = datetime.datetime.now()
+) -> value_objects.Result:
     if isinstance(job, job_spec.AdminJobSpec):
-        result: value_objects.Result = job.run(uow=uow, logger=logger)
+        return job.run(uow=uow, logger=logger) or value_objects.Result.success()
     elif isinstance(job, job_spec.ETLJobSpec):
-        result = job.run(logger=logger)
+        return job.run(logger=logger) or value_objects.Result.success()
     else:
         raise ValueError(
-            f"Expected an instance of AdminJobSpec or ETLJobSpec, but got {job}."
+            f"Expected an instance of AdminJobSpec or ETLJobSpec, but got {job!r}."
         )
-
-    if result is None:
-        result = value_objects.Result.success()
-    elif isinstance(result, value_objects.Result):
-        logger.log_info(
-            value_objects.LogMessage(f"[{job.job_name}] finished successfully.")
-        )
-    else:
-        raise ValueError(
-            f"A job should return an instance of either Success or Failure, but [{job.job_name}] "
-            f"returned {result!r}."
-        )
-
-    if result.is_failure:
-        logger.log_info(
-            value_objects.LogMessage(
-                f"[{job.job_name}] failed, so there is no point in running the tests.  "
-                f"They have been skipped."
-            )
-        )
-        test_results: typing.Collection[job_test_result.SimpleJobTestResult] = []
-    else:
-        logger.log_info(
-            value_objects.LogMessage(f"Running the tests for [{job.job_name}]...")
-        )
-        if isinstance(job, job_spec.AdminJobSpec):
-            test_results = job.test(logger=logger, uow=uow)
-        else:
-            test_results = job.test(logger=logger)  # type: ignore
-
-    if test_results:
-        full_test_results = frozenset(
-            job_test_result.JobTestResult(
-                id=value_objects.UniqueId.generate(),
-                job_id=job_id,
-                test_name=test_result.test_name,
-                test_success_or_failure=test_result.test_success_or_failure,
-                ts=ts_adapter.now(),
-            )
-            for test_result in test_results
-        )
-    else:
-        full_test_results = frozenset()
-
-    end_time = datetime.datetime.now()
-    execution_millis = int((end_time - start_time).total_seconds() * 1000)
-    ts = ts_adapter.now()
-    return job_result.JobResult(
-        id=job_id,
-        batch_id=batch_id,
-        job_name=job.job_name,
-        test_results=full_test_results,
-        execution_millis=value_objects.ExecutionMillis(execution_millis),
-        execution_success_or_failure=value_objects.Result.success(),
-        ts=ts,
-    )
